#
#  -*- coding: utf-8 -*-
#
import unittest
import os
import yaml
import tensorflow as tf
import numpy as np
from tensorflow.core.framework import graph_pb2
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import graph_util
from neural_compressor.adaptor.tf_utils.util import disable_random
from neural_compressor.utils.utility import CpuInfo
from neural_compressor.adaptor.tf_utils.graph_rewriter.graph_util import GraphRewriterHelper as Helper

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        graph_optimization:
          precisions: [bf16]
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            accuracy_criterion:
              relative: 0.0001
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()


def build_fake_yaml_2():
    fake_yaml_2 = '''
        model:
          name: fake_yaml_2
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        graph_optimization:
          precisions: [bf16]
        '''
    y = yaml.load(fake_yaml_2, Loader=yaml.SafeLoader)
    with open('fake_yaml_2.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()


def build_fake_yaml_3():
    fake_yaml_3 = '''
        model:
          name: fake_yaml_3
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        graph_optimization:
          precisions:
            - bf16
            - fp32
        '''
    y = yaml.load(fake_yaml_3, Loader=yaml.SafeLoader)
    with open('fake_yaml_3.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

def build_fake_yaml_4():
    fake_yaml_4 = '''
        model:
          name: fake_yaml_4
          framework: pytorch
          inputs: input
          outputs: op_to_store
        graph_optimization:
          precisions: [bf16]
        '''
    y = yaml.load(fake_yaml_4, Loader=yaml.SafeLoader)
    with open('fake_yaml_4.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

class MyMetric(object):
    def __init__(self, *args):
        self.pred_list = []
        self.label_list = []
        self.samples = 0
    def update(self, predict, label):
        self.pred_list.extend(predict)
        self.label_list.extend(label)
        self.samples += len(label)
    def reset(self):
        self.pred_list = []
        self.label_list = []
        self.samples = 0
    def result(self):
        pred = np.array(self.pred_list)
        label = np.array(self.label_list)
        ones = np.ones(pred.ndim, dtype=np.int)
        ones[0] = label.shape[0]
        label = np.array(self.label_list).reshape(ones)
        correct_num = np.sum(pred == label) 
        return correct_num / self.samples

class TestGraphOptimizationOnNonBF16Host(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        build_fake_yaml()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')

    @disable_random()
    def test_bf16_cfg_on_non_bf16_enabled_host(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 300, 300, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')
        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common
            from neural_compressor.conf.config import Graph_Optimization_Conf
            conf = Graph_Optimization_Conf('fake_yaml.yaml')
            graph_optimizer = Graph_Optimization(conf)
            dataset = graph_optimizer.dataset('dummy', shape=(100, 300, 300, 16), label=True)
            graph_optimizer.eval_dataloader = common.DataLoader(dataset)
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            if CpuInfo().bf16:
                self.assertEqual(found_cast_op, True)
            else:
                self.assertEqual(found_cast_op, False)

class TestGraphOptimization(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        os.environ['FORCE_BF16'] = '1'

        build_fake_yaml()
        build_fake_yaml_2()
        build_fake_yaml_3()
        build_fake_yaml_4()

    @classmethod
    def tearDownClass(self):
        del os.environ['FORCE_BF16']
        os.remove('fake_yaml.yaml')
        os.remove('fake_yaml_2.yaml')
        os.remove('fake_yaml_3.yaml')
        os.remove('fake_yaml_4.yaml')

    def test_not_supported_model(self):
        import torchvision
        model = torchvision.models.resnet18()
        from neural_compressor.experimental import Graph_Optimization
        graph_optimizer = Graph_Optimization('fake_yaml_4.yaml')
        graph_optimizer.input = 'input'
        graph_optimizer.output = 'op_to_store'
        graph_optimizer.model = model
        try:
            output_graph = graph_optimizer()
        except SystemExit:
            pass

    def test_not_supported_model_without_yaml(self):
        import torchvision
        model = torchvision.models.resnet18()
        from neural_compressor.experimental import Graph_Optimization
        graph_optimizer = Graph_Optimization()
        graph_optimizer.input = 'input'
        graph_optimizer.output = 'op_to_store'
        try:
            graph_optimizer.model = model
        except SystemExit:
            pass

    @disable_random()
    def test_graph_optimization_with_evaluation(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 300, 300, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')
        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common
            graph_optimizer = Graph_Optimization('fake_yaml.yaml')
            dataset = graph_optimizer.dataset('dummy', shape=(100, 300, 300, 16), label=True)
            graph_optimizer.eval_dataloader = common.DataLoader(dataset)
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_without_evaluation(self):

        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common
            graph_optimizer = Graph_Optimization('fake_yaml_2.yaml')
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_without_yaml(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')
        relu62 = tf.nn.relu6(conv_bias, name='op2_to_store')

        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[relu6.name.split(':')[0], relu62.name.split(':')[0]])

            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization()
            graph_optimizer.precisions = 'fp32'
            graph_optimizer.input = 'input'
            graph_optimizer.output = 'op_to_store, op2_to_store'

            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False
            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break
            input_name = graph_optimizer.input
            output_name = graph_optimizer.output
            self.assertEqual(found_cast_op, False)
            self.assertEqual(input_name, 'input')
            self.assertEqual(output_name, 'op_to_store, op2_to_store')

    @disable_random()
    def test_graph_optimization_with_yaml(self):

        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization('fake_yaml_3.yaml')

            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_with_custom_metric_without_postprocess(self):
        os.environ['FORCE_BF16'] = '1'

        x = tf.compat.v1.placeholder(tf.float32, [1, 300, 300, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')
        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common
            graph_optimizer = Graph_Optimization('fake_yaml_3.yaml')
            graph_optimizer.metric = common.Metric(MyMetric)
            dataset = graph_optimizer.dataset('dummy', shape=(100, 300, 300, 16), label=True)
            graph_optimizer.precisions = ['fp32', 'bf16']
            graph_optimizer.eval_dataloader = common.DataLoader(dataset)
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            self.assertIsNotNone(output_graph.graph_def)
            
            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_without_custom_metric_with_postprocess(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 300, 300, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')
        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common, data
            graph_optimizer = Graph_Optimization('fake_yaml.yaml')
            dataset = graph_optimizer.dataset('dummy', shape=(100, 300, 300, 16), label=True)
            graph_optimizer.eval_dataloader = common.DataLoader(dataset)
            graph_optimizer.postprocess = common.Postprocess(data.transforms.transform.TensorflowWrapFunction(np.array))
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_with_eval_func(self):

        x = tf.compat.v1.placeholder(tf.float32, [1, 300, 300, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization, common
            graph_optimizer = Graph_Optimization('fake_yaml.yaml')

            dataset = graph_optimizer.dataset('dummy', shape=(100, 300, 300, 16), label=True)
            graph_optimizer.eval_dataloader = common.DataLoader(dataset)
            graph_optimizer.model = output_graph_def
            graph_optimizer.eval_func = None

            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break
            self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_with_force_bf16(self):
        os.environ['FORCE_BF16'] = '1'

        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization()
            graph_optimizer.input = 'input'
            graph_optimizer.output = 'op_to_store'

            graph_optimizer.precisions = 'bf16'
            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

        self.assertEqual(found_cast_op, True)

    @disable_random()
    def test_graph_optimization_with_bn(self):
        input_constant_name = "input_constant"
        relu_name = "relu"
        float_graph_def = graph_pb2.GraphDef()
        input_constant = Helper.create_constant_node(
            input_constant_name,
            value=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
            dtype=dtypes.float32,
            shape=[1, 2, 6, 6])
        float_graph_def.node.extend([input_constant])
        relu_node = Helper.create_node("Relu", relu_name,
                                                    [input_constant_name])
        Helper.set_attr_dtype(relu_node, "T", dtypes.float32)
        float_graph_def.node.extend([relu_node])

        b_constant_name = "b_constant"
        conv2d_name = "conv2d_1"
        b_constant = Helper.create_constant_node(
            b_constant_name,
            value=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
            dtype=dtypes.float32,
            shape=[1, 2, 6, 6])
        float_graph_def.node.extend([b_constant])

        conv2d_node = Helper.create_node(
            "Conv2D", conv2d_name, [relu_name, b_constant_name])
        Helper.set_attr_dtype(conv2d_node, "T", dtypes.float32)
        Helper.set_attr_string(conv2d_node, "padding", b"SAME")
        Helper.set_attr_int_list(conv2d_node, "strides", [1,1,1,1])

        float_graph_def.node.extend([conv2d_node])

        bias_add_name = "bias_add"
        offset_constant_name = "offset_constant"

        offset_constant = Helper.create_constant_node(
            offset_constant_name,
            value=[1, 2, 3, 4, 5, 6],
            dtype=dtypes.float32,
            shape=[6])
        float_graph_def.node.extend([offset_constant])

        bias_add_node = Helper.create_node(
            "BiasAdd", bias_add_name, [conv2d_name, offset_constant_name])
        Helper.set_attr_dtype(bias_add_node, "T", dtypes.float32)
        float_graph_def.node.extend([bias_add_node])

        bn_scale_name = 'bn_scale'
        bn_scale_node = Helper.create_constant_node(
            bn_scale_name,
            value=[1, 2, 3, 4, 5, 6],
            dtype=dtypes.float32,
            shape=[6])
        bn_offset_name = 'bn_offset'
        bn_offset_node = Helper.create_constant_node(
            bn_offset_name,
            value=[1, 2, 3, 4, 5, 6],
            dtype=dtypes.float32,
            shape=[6])
        bn_mean_name = 'bn_mean'
        bn_mean_node = Helper.create_constant_node(
            bn_mean_name, value=[
                1,
                2,
            ], dtype=dtypes.float32, shape=[
                2,
            ])
        bn_var_name = 'bn_var'
        bn_var_node = Helper.create_constant_node(
            bn_var_name, value=[], dtype=dtypes.float32, shape=[0])
        fused_bn_node_name = 'bn'
        fused_bn_node = Helper.create_node(
            "FusedBatchNormV3", fused_bn_node_name, [
                bias_add_name, bn_scale_name, bn_offset_name, bn_mean_name,
                bn_var_name
            ])
        Helper.set_attr_dtype(fused_bn_node, "T", dtypes.float32)
        Helper.set_attr_dtype(fused_bn_node, "U", dtypes.float32)
        float_graph_def.node.extend([
            fused_bn_node, bn_scale_node, bn_offset_node, bn_mean_node,
            bn_var_node
        ])

        post_relu_name = "post_relu"
        post_relu_node = Helper.create_node(
            "Relu", post_relu_name, [fused_bn_node_name])
        Helper.set_attr_dtype(post_relu_node, "T", dtypes.float32)

        float_graph_def.node.extend([post_relu_node])

        from neural_compressor.experimental import Graph_Optimization

        graph_optimizer = Graph_Optimization()

        graph_optimizer.precisions = 'bf16'
        graph_optimizer.model = float_graph_def
        output_graph = graph_optimizer()
        bn_bf16 = False
        for i in output_graph.graph_def.node:
            if i.op == 'FusedBatchNormV3' and i.attr['T'].type == dtypes.bfloat16:
                bn_bf16 = True
        self.assertEqual(bn_bf16, True)
class TestGraphOptmizationFP32(unittest.TestCase):

    @disable_random()
    def test_graph_optimization_without_yaml_without_precisions(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization()
            graph_optimizer.input = 'input'
            graph_optimizer.output = 'op_to_store'

            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break
            precision = graph_optimizer.precisions
            self.assertEqual(found_cast_op, False)
            self.assertEqual(precision, 'fp32')

    @disable_random()
    def test_graph_optimization_without_yaml_with_precisions(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization()
            graph_optimizer.precisions = 'fp32'

            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, False)

    @disable_random()
    def test_graph_optimization_fp32_only_with_force_bf16(self):
        os.environ['FORCE_BF16'] = '1'

        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Graph_Optimization
            graph_optimizer = Graph_Optimization()
            graph_optimizer.input = 'input'
            graph_optimizer.output = 'op_to_store'

            graph_optimizer.model = output_graph_def
            output_graph = graph_optimizer()
            found_cast_op = False

            for i in output_graph.graph_def.node:
                if i.op == 'Cast':
                    found_cast_op = True
                    break

            self.assertEqual(found_cast_op, False)

if __name__ == "__main__":
    unittest.main()



"""Tests for model"""
import numpy as np
import unittest
import os

from neural_compressor.model import MODELS
import torchvision
import torch
import onnx
import mxnet.gluon.nn as nn
import mxnet as mx
import tensorflow as tf
import neural_compressor.model.model as NCModel
from neural_compressor.model.model import get_model_fwk_name
from neural_compressor.experimental.common.model import Model

def build_graph():
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with tf.Session(graph=graph) as sess:
            x = tf.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y = tf.constant(np.random.random((2, 2, 1, 1)), name='y')
            op = tf.nn.conv2d(input=x, filter=y, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session(graph=graph) as sess:
            x = tf.compat.v1.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y = tf.compat.v1.constant(np.random.random((3, 3, 1, 1)), name='y')
            op = tf.nn.conv2d(input=x, filters=y, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')
            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    return graph
        
def build_estimator():
    def model_fn(features, labels, mode):
        logits = tf.keras.layers.Dense(12)(features)
        logits = tf.keras.layers.Dense(56)(logits)
        logits = tf.keras.layers.Dense(4)(logits)
    
        output_spec = tf.estimator.EstimatorSpec(
            mode=tf.estimator.ModeKeys.PREDICT, predictions=logits)
        return output_spec
    return model_fn

def build_input_fn():
    def input_fun():
        tf.compat.v1.disable_eager_execution()
        raw_dataset = np.ones([100,224, 224, 3], dtype=np.float32)
        tf_dataset = tf.compat.v1.data.Dataset.from_tensor_slices(raw_dataset)
        tf_dataset = tf_dataset.batch(1)
        ds_iterator = tf_dataset.make_initializable_iterator()
        iter_tensors = ds_iterator.get_next()
        return iter_tensors
    return input_fun

def build_keras():
    from tensorflow import keras
    (train_images, train_labels), (test_images,
                                    test_labels) = keras.datasets.fashion_mnist.load_data()

    train_images = train_images.astype(np.float32) / 255.0

    # Create Keras model
    model = keras.Sequential([
      keras.layers.InputLayer(input_shape=(28, 28), name="input"),
      keras.layers.Reshape(target_shape=(28, 28, 1)),
      keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
      keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
      keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
      keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
      keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
      keras.layers.MaxPooling2D(pool_size=(2, 2)),
      keras.layers.Flatten(),
      keras.layers.Dense(10, activation="softmax", name="output")
    ])
   
    # Compile model with optimizer
    opt = keras.optimizers.Adam(learning_rate=0.01)
    model.compile(optimizer=opt,
                   loss="sparse_categorical_crossentropy",
                   metrics=["accuracy"])

    # # Train model
    model.fit(\
        x={"input": train_images[0:100]}, y={"output": train_labels[0:100]}, epochs=1)
    return model

class TestTensorflowModel(unittest.TestCase):

    @classmethod
    def tearDownClass(self):
        os.remove('model_test.pb')

    def test_graph(self):
        graph = build_graph()
        model = Model(graph)
        model.input_tensor_names = ['x']
        model.output_tensor_names = ['op_to_store']

        self.assertEqual(True, isinstance(model.graph_def, tf.compat.v1.GraphDef))
        self.assertEqual(model.input_node_names[0], 'x')
        self.assertEqual(model.output_node_names[0], 'op_to_store')
        model.save('model_test.pb')

        model = Model('model_test.pb')
        self.assertEqual(model.input_tensor_names[0], 'x')
        self.assertEqual(model.output_tensor_names[0], 'op_to_store')
        self.assertEqual(model.input_tensor[0].name, 'x:0')
        self.assertEqual(model.output_tensor[0].name, 'op_to_store:0')

        # test wrong input tensor names can't set
        with self.assertRaises(AssertionError):
            model.input_tensor_names = ['wrong_input']
        with self.assertRaises(AssertionError):
            model.output_tensor_names = ['wrong_output']

        # test right tensor
        model.input_tensor_names = ['x_1']
        model.output_tensor_names = ['op_to_store_1']
        self.assertEqual(True, isinstance(model.graph_def, tf.compat.v1.GraphDef))

    def test_validate_graph_node(self):
        from neural_compressor.model.model import validate_graph_node
        graph = build_graph()
        self.assertEqual(False, validate_graph_node(graph.as_graph_def(), []))
        self.assertEqual(False, validate_graph_node(graph.as_graph_def(), ['test']))
        self.assertEqual(True, validate_graph_node(graph.as_graph_def(), ['x']))

    def test_estimator(self):
        from neural_compressor.adaptor.tf_utils.util import get_estimator_graph
        model_fn = build_estimator()
        input_fn = build_input_fn() 
        estimator = tf.estimator.Estimator(
            model_fn, model_dir=None, config=None, params=None, warm_start_from=None
            )
        with self.assertRaises(AssertionError):
            graph_def = Model(estimator).graph_def
        model = Model(estimator, input_fn=input_fn)
        self.assertEqual(model.output_tensor_names[0], 'dense_2/BiasAdd:0')

    def test_ckpt(self):
        mobilenet_ckpt_url = \
            'http://download.tensorflow.org/models/mobilenet_v1_2018_02_22/mobilenet_v1_1.0_224.tgz'
        dst_path = '/tmp/.neural_compressor/mobilenet_v1_1.0_224.tgz'
        if not os.path.exists(dst_path):
          os.system("mkdir -p /tmp/.neural_compressor && wget {} -O {}".format(
                  mobilenet_ckpt_url, dst_path))

        os.system("mkdir -p ckpt && tar xvf {} -C ckpt".format(dst_path))
        model = Model('./ckpt')
        model.output_tensor_names = ['MobilenetV1/Predictions/Reshape_1']
        
        self.assertGreaterEqual(len(model.input_tensor_names), 1)
        self.assertEqual(len(model.output_tensor_names), 1)
        graph_def = model.graph_def
        self.assertEqual(True, isinstance(graph_def, tf.compat.v1.GraphDef))
        model.graph_def = graph_def
        os.system('rm -rf ckpt')

    def test_slim(self):
        tf.compat.v1.reset_default_graph()
        inception_ckpt_url = \
            'http://download.tensorflow.org/models/inception_v1_2016_08_28.tar.gz'
        dst_path = '/tmp/.neural_compressor/slim/inception_v1_2016_08_28.tar.gz'
        if not os.path.exists(dst_path):
            os.system("mkdir -p /tmp/.neural_compressor/slim")
            os.system("wget {} -O {}".format(inception_ckpt_url, dst_path))

        os.system("mkdir -p slim_ckpt && tar xvf {} -C slim_ckpt".format(dst_path))
        if tf.version.VERSION > '2.0.0':
            return
        from tf_slim.nets import inception  
        model = Model('./slim_ckpt/inception_v1.ckpt')
        model.name = 'inception_v1'
        graph_def = model.graph_def
        self.assertGreaterEqual(len(model.output_node_names), 1)
        self.assertGreaterEqual(len(model.input_node_names), 1)
        # test net factory
        from neural_compressor.model.nets_factory import TFSlimNetsFactory
        factory = TFSlimNetsFactory()
        from tf_slim.nets import inception
        input_shape = [None, 224, 224, 3] 
        model_func = inception.inception_v1
        arg_scope = inception.inception_v1_arg_scope
        num_classes = 1001
        factory.register('inceptionv1', model_func, input_shape, \
            arg_scope, num_classes=num_classes)
        os.system('rm -rf slim_ckpt')

    def test_keras_saved_model(self):
        if tf.version.VERSION < '2.3.0':
            return
        keras_model = build_keras()
        self.assertEqual('tensorflow', get_model_fwk_name(keras_model))

        model = Model(keras_model)
        self.assertGreaterEqual(len(model.output_node_names), 1)
        self.assertGreaterEqual(len(model.input_node_names), 1)
        keras_model.save('./simple_model')
        # load from path
        model = Model('./simple_model')
        self.assertGreaterEqual(len(model.output_node_names), 1)
        self.assertGreaterEqual(len(model.input_node_names), 1)

        os.makedirs('./keras_model', exist_ok=True)
        model.save('./keras_model')
        os.system('rm -rf simple_model')
        os.system('rm -rf keras_model')

    @unittest.skipIf(tf.version.VERSION < '2.4.0', "Only supports tf 2.4.0 or above")
    def test_saved_model(self):
        ssd_resnet50_ckpt_url = 'http://download.tensorflow.org/models/object_detection/ssd_resnet50_v1_fpn_shared_box_predictor_640x640_coco14_sync_2018_07_03.tar.gz'
        center_resnet50_saved_model_url = 'https://gcs.tensorflow.google.cn/tfhub-modules/tensorflow/centernet/resnet50v1_fpn_512x512/1.tar.gz'
        dst_path = '/tmp/.neural_compressor/saved_model.tar.gz'
        center_dst_path = '/tmp/.neural_compressor/center_saved_model.tar.gz'
        if not os.path.exists(dst_path):
          os.system("mkdir -p /tmp/.neural_compressor && wget {} -O {}".format(ssd_resnet50_ckpt_url, dst_path))
        if not os.path.exists(center_dst_path):
          os.system("mkdir -p /tmp/.neural_compressor && wget {} -O {}".format(center_resnet50_saved_model_url, center_dst_path))
        os.system("tar -xvf {}".format(dst_path))
        unzip_center_model = 'unzip_center_model'
        os.system("mkdir -p {} ".format(unzip_center_model))
        os.system("tar -xvf {} -C {}".format(center_dst_path,unzip_center_model))
        model = Model('ssd_resnet50_v1_fpn_shared_box_predictor_640x640_coco14_sync_2018_07_03/saved_model')
        center_model = Model('unzip_center_model')
        from tensorflow.python.framework import graph_util  
        graph_def = graph_util.convert_variables_to_constants(
            sess=model.sess,
            input_graph_def=model.graph_def,
            output_node_names=model.output_node_names)
     
        model.graph_def = graph_def
        tmp_saved_model_path = './tmp_saved_model'

        if os.path.exists(tmp_saved_model_path):
           os.system('rm -rf {}'.format(tmp_saved_model_path))
        os.system('mkdir -p {}'.format(tmp_saved_model_path))
       
        self.assertTrue(isinstance(model.graph_def, tf.compat.v1.GraphDef))
        self.assertTrue(isinstance(model.graph, tf.compat.v1.Graph))
        model.save(tmp_saved_model_path)
        # load again to make sure model can be loaded
        model = Model(tmp_saved_model_path)
        os.system('rm -rf ssd_resnet50_v1_fpn_shared_box_predictor_640x640_coco14_sync_2018_07_03')
        os.system('rm -rf temp_saved_model')
        os.system('rm -rf {}'.format(tmp_saved_model_path))
 
        center_graph_def = graph_util.convert_variables_to_constants(
            sess=center_model.sess,
            input_graph_def=center_model.graph_def,
            output_node_names=center_model.output_node_names)
     
        center_model.graph_def = center_graph_def
       
        self.assertTrue(isinstance(center_model.graph_def, tf.compat.v1.GraphDef))
        self.assertTrue(isinstance(center_model.graph, tf.compat.v1.Graph))
        os.system('rm -rf unzip_center_model')


    def test_tensorflow(self):
        from neural_compressor.model.model import TensorflowBaseModel
        ori_model = build_graph()
        self.assertEqual('tensorflow', get_model_fwk_name(ori_model))
        self.assertEqual('tensorflow', get_model_fwk_name(TensorflowBaseModel(ori_model)))
        try:
            get_model_fwk_name([])
        except AssertionError:
            pass
        try:
            get_model_fwk_name('./model.pb')
        except AssertionError:
            pass

def export_onnx_model(model, path):
    x = torch.randn(100, 3, 224, 224, requires_grad=True)
    torch_out = model(x)
    torch.onnx.export(model,
                      x,
                      path,
                      export_params=True,
                      opset_version=11,
                      do_constant_folding=True,
                      input_names = ["input"],
                      output_names = ["output"],
                      dynamic_axes={"input" : {0 : "batch_size"},
                                    "output" : {0 : "batch_size"}})

class TestONNXModel(unittest.TestCase):
    cnn_export_path = "cnn.onnx"
    cnn_model = torchvision.models.quantization.resnet18()
    
    @classmethod
    def setUpClass(self):
        cnn_model = torchvision.models.quantization.resnet18()
        export_onnx_model(self.cnn_model, self.cnn_export_path)
        self.cnn_model = onnx.load(self.cnn_export_path)

    @classmethod
    def tearDownClass(self):
        os.remove(self.cnn_export_path)

    def test_model(self):
        self.assertEqual('onnxruntime', get_model_fwk_name(self.cnn_export_path))
        model = MODELS['onnxruntime'](self.cnn_model)
        self.assertEqual(True, isinstance(model, NCModel.ONNXModel))
        self.assertEqual(True, isinstance(model.model, onnx.ModelProto))

        model.save('test.onnx')
        self.assertEqual(True, os.path.exists('test.onnx'))
        os.remove('test.onnx')

class TestPyTorchModel(unittest.TestCase):
    def testPyTorch(self):
        import torchvision
        from neural_compressor.model.model import PyTorchModel, PyTorchIpexModel, PyTorchFXModel
        ori_model = torchvision.models.mobilenet_v2()
        self.assertEqual('pytorch', get_model_fwk_name(ori_model))
        pt_model = PyTorchModel(ori_model)
        pt_model.model = ori_model
        pt_model = PyTorchModel(torchvision.models.mobilenet_v2())
        with self.assertRaises(AssertionError):
            pt_model.workspace_path = './pytorch'
        
        ipex_model = PyTorchIpexModel(ori_model)
        self.assertTrue(ipex_model.model)
        ipex_model.model = ori_model
        ipex_model = PyTorchModel(torchvision.models.mobilenet_v2())
        with self.assertRaises(AssertionError):
            ipex_model.workspace_path = './pytorch'
        ipex_model.save('./')

        self.assertEqual('pytorch', get_model_fwk_name(PyTorchModel(ori_model)))
        self.assertEqual('pytorch', get_model_fwk_name(PyTorchIpexModel(ori_model)))
        self.assertEqual('pytorch', get_model_fwk_name(PyTorchFXModel(ori_model)))

def load_mxnet_model(symbol_file, param_file):
    symbol = mx.sym.load(symbol_file)
    save_dict = mx.nd.load(param_file)
    arg_params = {}
    aux_params = {}
    for k, v in save_dict.items():
        tp, name = k.split(':', 1)
        if tp == 'arg':
            arg_params[name] = v
    return symbol, arg_params, aux_params

class TestMXNetModel(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        net = nn.HybridSequential()
        net.add(nn.Dense(128, activation="relu"))
        net.add(nn.Dense(64, activation="relu"))
        net.add(nn.Dense(10))
        net.initialize()
        net.hybridize()
        fake_data = mx.random.uniform(shape=(1,128,128))
        net(fake_data)
        self.net = net

    @classmethod
    def tearDownClass(self):
        os.remove('test-symbol.json')
        os.remove('test-0000.params')
        os.remove('test2-symbol.json')
        os.remove('test2-0000.params')

    def test_model(self):
        self.assertEqual('mxnet', get_model_fwk_name(self.net))
        model = MODELS['mxnet'](self.net)
        self.assertEqual(True, isinstance(model, NCModel.MXNetModel))
        self.assertEqual(True, isinstance(model.model, mx.gluon.HybridBlock))

        model.save('./test')
        self.assertEqual(True, os.path.exists('test-symbol.json'))
        self.assertEqual(True, os.path.exists('test-0000.params'))

        net = load_mxnet_model('test-symbol.json', 'test-0000.params')
        model.model = net
        self.assertEqual(True, isinstance(model.model[0], mx.symbol.Symbol))
        model.save('./test2')
        self.assertEqual(True, os.path.exists('test2-symbol.json'))
        self.assertEqual(True, os.path.exists('test2-0000.params'))

if __name__ == "__main__":
    unittest.main()



#
#  -*- coding: utf-8 -*-
#
import unittest
import os
from neural_compressor.adaptor.tensorflow import TensorFlowAdaptor
from neural_compressor.experimental.common.model import Model as TensorflowModel
from neural_compressor.model.model import validate_graph_node

class TestTFAutoDetectInputOutput(unittest.TestCase):
    mb_model_url = 'https://storage.googleapis.com/intel-optimized-tensorflow/models/v1_6/mobilenet_v1_1.0_224_frozen.pb'
    pb_path = '/tmp/.neural_compressor/mobilenet_fp32.pb'

    @classmethod
    def setUpClass(self):
        self.saved_flag = True
        if not os.path.exists(self.pb_path):
            try:
                os.system("mkdir -p /tmp/.neural_compressor && wget {} -O {} ".format(self.mb_model_url, self.pb_path))
            except Exception as e:
                self.saved_flag = False

    def testAutoDetectInputOutput(self):
        if self.saved_flag:
            model = TensorflowModel(self.pb_path)
            outputs = model.output_node_names
            inputs = model.input_node_names
            output_validate = validate_graph_node(model.graph_def, outputs)
            self.assertTrue(output_validate)

            input_validate = validate_graph_node(model.graph_def, inputs)
            self.assertTrue(input_validate)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Optimization class."""

import os
import re
from abc import abstractmethod
from typing import Any, Dict, List, Optional

from neural_compressor.ux.utils.exceptions import ClientErrorException, InternalException
from neural_compressor.ux.utils.hw_info import HWInfo
from neural_compressor.ux.utils.json_serializer import JsonSerializer
from neural_compressor.ux.utils.workload.workload import Workload


class Optimization(JsonSerializer):
    """Optimization class."""

    def __init__(
        self,
        workload: Workload,
        template_path: Optional[str] = None,
    ) -> None:
        """Initialize Optimization class."""
        self.input_graph: str = workload.model_path
        self.input_precision: str = workload.input_precision
        self.output_graph: str = workload.model_output_path
        self.output_precision: str = workload.output_precision
        self.framework: str = workload.framework

        self.input_nodes: Optional[str] = None
        self.output_nodes: Optional[str] = None
        self.set_boundary_nodes(workload)

        self.tune: bool = workload.tune
        self.config_path = workload.config_path

        self.instances: int = 1
        self.cores_per_instance: int = HWInfo().cores // self.instances

        if not os.path.exists(self.input_graph):
            raise ClientErrorException("Could not found model in specified path.")

        self._command: Optional[List[str]] = None

    def execute(self) -> None:
        """Execute optimization."""
        raise NotImplementedError

    @property
    def command(self) -> List[str]:
        """Get execution command."""
        if self._command is None:
            if isinstance(self.optimization_script, str) and isinstance(self.parameters, list):
                self._command = ["python", self.optimization_script] + self.parameters
            else:
                raise NotImplementedError
        return self._command

    @command.setter
    def command(self, value: List[str]) -> None:
        """Set optimization command."""
        if not isinstance(value, list):
            raise InternalException("Command should be a list.")
        self._command = value

    @property
    @abstractmethod
    def optimization_script(self) -> str:
        """Get optimization script path."""
        raise NotImplementedError

    @property
    @abstractmethod
    def parameters(self) -> List[str]:
        """Get optimization parameters."""
        raise NotImplementedError

    def serialize(self, serialization_type: str = "default") -> Dict[str, Any]:
        """Serialize Optimization class to dict."""
        result = {}
        for key, value in self.__dict__.items():
            variable_name = re.sub(r"^_", "", key)
            if variable_name == "command":
                result[variable_name] = " ".join(value)
            else:
                result[variable_name] = value
        return result

    def set_boundary_nodes(self, workload: Workload) -> None:
        """Set optimization input and output nodes according to workload."""
        if isinstance(workload.input_nodes, str):
            self.input_graph = workload.input_nodes
        if isinstance(workload.input_nodes, list):
            self.input_nodes = ",".join(workload.input_nodes)

        if isinstance(workload.output_nodes, str):
            self.output_nodes = workload.output_nodes
        if isinstance(workload.output_nodes, list):
            self.output_nodes = ",".join(workload.output_nodes)



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Execute tune."""

import json
import os
import threading
from typing import Any, Dict, Optional

from neural_compressor.ux.components.optimization.factory import OptimizationFactory
from neural_compressor.ux.components.optimization.optimization import Optimization
from neural_compressor.ux.components.optimization.tuning_history import Watcher
from neural_compressor.ux.utils.exceptions import ClientErrorException
from neural_compressor.ux.utils.executor import Executor
from neural_compressor.ux.utils.logger import log
from neural_compressor.ux.utils.parser import OptimizationParser
from neural_compressor.ux.utils.templates.workdir import Workdir
from neural_compressor.ux.utils.utils import _load_json_as_dict, get_size
from neural_compressor.ux.utils.workload.workload import ExecutionMode
from neural_compressor.ux.web.communication import MessageQueue

mq = MessageQueue()


def execute_optimization(data: Dict[str, Any]) -> dict:
    """Get configuration."""
    from neural_compressor.ux.utils.workload.workload import Workload

    if not str(data.get("id", "")):
        message = "Missing request id."
        mq.post_error(
            "optimization_finish",
            {"message": message, "code": 404},
        )
        raise Exception(message)

    request_id: str = data["id"]

    workdir: Optional[Workdir] = None

    try:
        workdir = Workdir(request_id=request_id, overwrite=False)
        workload_path: str = workdir.workload_path
        workload_data = _load_json_as_dict(
            os.path.join(workload_path, "workload.json"),
        )
        workload = Workload(workload_data)
        optimization: Optimization = OptimizationFactory.get_optimization(
            workload,
            workdir.template_path,
        )
        send_data = {
            "message": "started",
            "id": request_id,
            "size_input_model": get_size(optimization.input_graph),
        }
        workdir.clean_logs()
        workdir.update_data(
            request_id=request_id,
            model_path=optimization.input_graph,
            input_precision=optimization.input_precision,
            model_output_path=optimization.output_graph,
            output_precision=optimization.output_precision,
            status="wip",
        )

        executor = Executor(
            workspace_path=workload_path,
            subject="optimization",
            data=send_data,
            log_name="output",
        )

        tuning_history_watcher = Watcher(request_id)
        threading.Thread(target=tuning_history_watcher, daemon=True).start()

        proc = executor.call(
            optimization.command,
        )

        tuning_history_watcher.stop()

        optimization_time = executor.process_duration
        if optimization_time:
            optimization_time = round(optimization_time, 2)
        log.debug(f"Elapsed time: {optimization_time}")
        logs = [os.path.join(workload_path, "output.txt")]
        parser = OptimizationParser(logs)
        if proc.is_ok:
            response_data = parser.process()

            if ExecutionMode.BASIC == workload.execution_mode:
                response_data.update(
                    {
                        "acc_input_model": None,
                        "acc_optimized_model": None,
                    },
                )

            optimized_model_path = response_data.get(
                "path_optimized_model",
                optimization.output_graph,
            )
            optimization.output_graph = optimized_model_path

            if isinstance(response_data, dict):
                response_data["id"] = request_id
                response_data["optimization_time"] = optimization_time
                response_data["size_optimized_model"] = get_size(optimized_model_path)
                response_data["model_output_path"] = optimized_model_path
                response_data["size_input_model"] = get_size(optimization.input_graph)
                response_data["is_custom_dataloader"] = bool(workdir.template_path)

                workdir.update_data(
                    request_id=request_id,
                    model_path=optimization.input_graph,
                    model_output_path=optimized_model_path,
                    metric=response_data,
                    status="success",
                    execution_details={"optimization": optimization.serialize()},
                    input_precision=optimization.input_precision,
                    output_precision=optimization.output_precision,
                )
                response_data["execution_details"] = {"optimization": optimization.serialize()}

            log.debug(f"Parsed data is {json.dumps(response_data)}")
            mq.post_success("optimization_finish", response_data)
            return response_data
        else:
            log.debug("FAIL")
            workdir.update_data(
                request_id=request_id,
                model_path=optimization.input_graph,
                input_precision=optimization.input_precision,
                output_precision=optimization.output_precision,
                status="error",
            )
            raise ClientErrorException("Optimization failed during execution.")
    except Exception as err:
        mq.post_failure("optimization_finish", {"message": str(err), "id": request_id})
        if workdir is not None and workdir.get_workload_data(request_id).get("status") != "error":
            workdir.update_data(
                request_id=request_id,
                status="error",
            )
        raise



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod
from collections import namedtuple, OrderedDict
from .tensor import Tensor
from ..tf_utils import tf_extract_operator
from ..onnx_utils import onnx_extract_operator

OP_EXTRACTORS = {
    'tensorflow': tf_extract_operator,
    'onnxruntime': onnx_extract_operator,
}

OPERATORS = {}


def operator_registry(operator_type):
    """The class decorator used to register all Algorithm subclasses.

    Args:
        cls (class): The class of register.
        operator_type (str): The operator registration name

    Returns:
        cls: The class of register.
    """
    def decorator_operator(cls):
        if operator_type in OPERATORS:
            raise ValueError('Cannot have two operators with the same name')
        OPERATORS[operator_type] = cls
        return cls

    return decorator_operator


class Operator(object):
    def __init__(self):
        self._name = ''
        self._op_type = ''
        self._input_tensors = []
        self._output_tensors = []
        self._attr = OrderedDict()

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def op_type(self):
        return self._op_type

    @op_type.setter
    def op_type(self, op_type):
        self._op_type = op_type

    @property
    def input_tensors(self):
        return self._input_tensors

    @input_tensors.setter
    def input_tensors(self, input_tensors):
        self._input_tensors = input_tensors

    @property
    def output_tensors(self):
        return self._output_tensors

    @output_tensors.setter
    def output_tensors(self, output_tensors):
        self._output_tensors = output_tensors

    @property
    def attr(self):
        return self._attr

    @attr.setter
    def attr(self, attr):
        self._attr = attr

    def set_attr(self, framework, node):
        self._attr = OrderedDict()

    # extract the op from framework
    def extract(self, framework, node, model, nodes_dict):
        self._name = node.name
        self._op_type, self._input_tensors, self._output_tensors = OP_EXTRACTORS[framework](
            node, model, nodes_dict)
        self.set_attr(framework, node)

    # make the op by set the attributes
    def construct(self, name, op_type, input_tensors=[], output_tensors=[], attr=OrderedDict()):
        self._name = name
        self._op_type = op_type
        self._input_tensors = input_tensors
        self._output_tensors = output_tensors
        self._attr = attr

    # get the op config in graph
    @property
    def config(self):
        conf_dict = OrderedDict()
        # conf_dict['type'] = self._op_type
        conf_dict['type'] = self._op_type
        if len(self._input_tensors) > 0:
            conf_dict['input'] = OrderedDict()
            for input_tensor in self._input_tensors:
                if self._op_type == 'Input':
                    conf_dict['input'][input_tensor.name] = input_tensor.config
                else:
                    conf_dict['input'][input_tensor.name] = {}
        if len(self._output_tensors) > 0:
            conf_dict['output'] = OrderedDict()
            for output_tensor in self._output_tensors:
                if self._op_type == 'Input':
                    conf_dict['output'][output_tensor.name] = output_tensor.config
                else:
                    conf_dict['output'][output_tensor.name] = {}

        if isinstance(self._attr, dict) and len(self._attr.keys()) > 0:
            conf_dict['attr'] = self._attr

        return conf_dict



import unittest
import os
import sys
import shutil
import yaml
import json
import numpy as np
import mxnet as mx
import mxnet.gluon.nn as nn
from pathlib import Path

from neural_compressor.experimental import Quantization, common
from neural_compressor.utils.utility import recover


sys.path.append('..')

WORKSPACE_DIR = Path('./saved')


def get_mlp_sym():
    data = mx.symbol.Variable('data')
    out = mx.symbol.FullyConnected(data=data, name='fc1', num_hidden=1000)
    out = mx.symbol.Activation(data=out, act_type='relu')
    out = mx.symbol.Softmax(out, name='softmax')
    return out


def get_conv_sym():
    data = mx.sym.Variable('data')
    conv = mx.sym.Convolution(data, kernel=(1, 1), num_filter=3, name='conv')
    batch_norm = mx.sym.BatchNorm(data=conv, eps=2e-05, fix_gamma=False,
                                  momentum=0.9, use_global_stats=False, name='bn')
    act = mx.sym.Activation(data=batch_norm, act_type='relu', name='relu')
    pool = mx.sym.Pooling(act, kernel=(4, 4), pool_type='avg', name='pool')
    out = mx.sym.FullyConnected(pool, num_hidden=10, flatten=True, name='fc')

    return out


def build_mxnet():
    fake_yaml = '''
        model:
          name: imagenet
          framework: mxnet

        evaluation:
          accuracy:
            metric:
              topk: 1

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
          workspace:
            path: {}
        '''.format(str(WORKSPACE_DIR))
    configs = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('mxnet.yaml', "w", encoding="utf-8") as f:
        yaml.dump(configs, f)
    f.close()


def build_mxnet_kl():
    fake_yaml = '''
        model:
          name: imagenet
          framework: mxnet

        evaluation:
          accuracy:
            metric:
              topk: 1

        quantization:
          model_wise:
            activation:
              algorithm: kl

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
          workspace:
            path: {}
        '''.format(str(WORKSPACE_DIR))
    configs = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('mxnet_kl.yaml', "w", encoding="utf-8") as f:
        yaml.dump(configs, f)
    f.close()


def are_models_equal(tester, model_a, model_b):
  symnet_a, args_a, auxs_a = model_a
  symnet_b, args_b, auxs_b = model_b

  nodes_a = {node['name']: node['op'] for node in json.loads(model_a[0].tojson())['nodes']}
  nodes_b = {node['name']: node['op'] for node in json.loads(model_b[0].tojson())['nodes']}
  tester.assertEqual(nodes_a, nodes_b)

  tester.assertEqual(args_a.keys(), args_b.keys())
  for key in args_a.keys():
    tester.assertTrue(np.all((args_a[key] == args_b[key]).asnumpy()))

  tester.assertEqual(auxs_a.keys(), auxs_b.keys())
  for key in auxs_a.keys():
    tester.assertTrue(np.all((auxs_a[key] == auxs_b[key]).asnumpy()))


class TestAdaptorMXNet(unittest.TestCase):
    """
    Test MXNet adaptor functions.
    """
    @classmethod
    def setUpClass(self):
        build_mxnet()
        build_mxnet_kl()
        self.mlp_model = get_mlp_sym()
        self.conv_model = get_conv_sym()
        self.quantizer_1 = Quantization("./mxnet.yaml")
        self.quantizer_2 = Quantization("./mxnet_kl.yaml")

        self.data_low = -1000
        self.data_high = 1000

    @classmethod
    def tearDownClass(self):
        os.remove('mxnet.yaml')
        os.remove('mxnet_kl.yaml')
        shutil.rmtree(WORKSPACE_DIR, ignore_errors=True)
        shutil.rmtree('runs', ignore_errors=True)

    def test_utils(self):
      import neural_compressor.adaptor.mxnet_utils.util as utils
      self.assertTrue(utils.isiterable([1, 2, 3]))
      self.assertFalse(utils.isiterable(123))



    def test_mlp_model_quantization(self):
        """
        Use MLP model to test minmax calibration and built-in evaluate function.
        """
        for shape in [(500, 1000), ]:
            arg_shapes, label_shape, _ = self.mlp_model.infer_shape(data=shape)

            mod = mx.mod.Module(symbol=self.mlp_model, context=mx.current_context())
            mod.bind(for_training=False,
                     data_shapes=[('data', arg_shapes[0])],
                     label_shapes=[('softmax_label', label_shape[0])])
            mod.init_params()

            arg_params, aux_params = mod.get_params()
            data = mx.nd.random.uniform(low=self.data_low, high=self.data_high,
                                        shape=shape).astype('float32')
            labels = mx.nd.ones([shape[0], ])
            calib_data = mx.io.NDArrayIter(data=data, label=labels, batch_size=shape[0])

            fp32_model = (self.mlp_model, arg_params, aux_params)
            self.quantizer_1.model = common.Model(fp32_model)
            self.quantizer_1.calib_dataloader = calib_data
            self.quantizer_1.eval_dataloader = calib_data
            qmodel = self.quantizer_1()
            self.assertIsInstance(qmodel.model[0], mx.symbol.Symbol)

    def test_conv_model_quantization(self):
        """
        Use Conv model to test KL calibration and user specific evaluate function.
        """
        for shape in [(32, 3, 224, 224), ]:
            arg_shapes, _, _ = self.conv_model.infer_shape(data=shape)

            mod = mx.mod.Module(symbol=self.conv_model, context=mx.current_context())
            mod.bind(for_training=False,
                     data_shapes=[('data', arg_shapes[0])])
            mod.init_params()

            arg_params, aux_params = mod.get_params()
            dataShape = (shape[0]*5, *shape[1:])
            data = mx.nd.random.uniform(low=self.data_low, high=self.data_high,
                                        shape=dataShape, dtype='float32')
            label = mx.nd.random.randint(low=0, high=1, shape=(dataShape[0])).astype('float32')
            calib_data = mx.io.NDArrayIter(data=data, label=label,  batch_size=shape[0])

            fp32_model = (self.conv_model, arg_params, aux_params)
            self.quantizer_2.model = common.Model(fp32_model)
            self.quantizer_2.calib_dataloader = calib_data
            self.quantizer_2.eval_dataloader = calib_data
            qmodel = self.quantizer_2()
            self.assertIsInstance(qmodel.model[0], mx.symbol.Symbol)

            # test inspected_tensor
            inspect_tensor = self.quantizer_2.strategy.adaptor.inspect_tensor
            self.quantizer_2.model = fp32_model
            insp = inspect_tensor(self.quantizer_2.model, calib_data,
                                  op_list=[('sg_mkldnn_conv_bn_act_0', 'sg_mkldnn_conv'),
                                           ('data', 'input')],
                                  iteration_list=[1, 3])
            qinsp = inspect_tensor(qmodel, calib_data,
                                   op_list=[('sg_mkldnn_conv_bn_act_0', 'sg_mkldnn_conv')],
                                   iteration_list=[1, 3])

            self.assertNotEqual(len(insp['activation']), 0)
            self.assertEqual(len(insp['activation']), len(qinsp['activation']))

            for tensors, qtensors in zip(insp['activation'], qinsp['activation']):
              for k in (set(tensors.keys()) & set(qtensors.keys())):
                tensor, qtensor = tensors[k][k[0]], qtensors[k][k[0]]
                self.assertEqual(tensor.shape, qtensor.shape)

            #test inspect with an empty iteration_list
            inspect_tensor(qmodel, calib_data,
                           op_list=[('sg_mkldnn_conv_bn_act_0', 'sg_mkldnn_conv')],
                           iteration_list=[])

            # test recovery for symbolic model
            qmodel_r = recover(fp32_model, WORKSPACE_DIR/'history.snapshot', -1)
            are_models_equal(self, qmodel.model, qmodel_r.model)

            # test symbolic model saving
            qmodel_r.save(WORKSPACE_DIR/'save_test')

    def test_gluon_model(self):
        """
        Use gluon model to test gluon related functions in mxnet adaptor.
        """
        # create gluon model
        def create_model(params=None):
          net = nn.HybridSequential()
          net.add(nn.Conv2D(1, (1, 1), activation="relu", prefix='conv1'))
          net.add(nn.Flatten(prefix='flatten1'))
          net.add(nn.Dense(64, activation="relu", prefix='dense1'))
          net.add(nn.Dense(10, prefix='dense2'))
          if params is not None:
            net.collect_params().load_dict({k: v.data() for k, v in params.items()})
          else:
            net.initialize()
          return net

        class Quant_dataloader():
            def __init__(self, dataset, batch_size=1):
                self.dataset = dataset
                self.batch_size = batch_size
                self._iter = None

            def __iter__(self):
              self._iter = iter(self.dataset)
              return self

            def __next__(self):
              data, label = self._iter.__next__()
              data = mx.nd.reshape(data, (1, data.shape[-1], *data.shape[:-1])).astype('float32')
              return (data, mx.nd.array(label, dtype='float32'))

        net = create_model()
        valid_dataset = mx.gluon.data.vision.datasets.FashionMNIST(train=False)
        q_dataloader = Quant_dataloader(valid_dataset)
        self.quantizer_1.model = common.Model(net)
        self.quantizer_1.calib_dataloader = q_dataloader
        qmodel = self.quantizer_1()
        self.assertIsInstance(qmodel.model, mx.gluon.HybridBlock)

        # test recovery for gluon model
        net = create_model(net.collect_params())
        qmodel_r = recover(net, WORKSPACE_DIR/'history.snapshot', -1)

        from neural_compressor.adaptor.mxnet_utils.util import prepare_model, prepare_dataloader
        dataloader = prepare_dataloader(qmodel, mx.cpu(), q_dataloader)

        # test calling prepare_dataloader for already prepared dataloader
        self.assertIs(dataloader, prepare_dataloader(qmodel, mx.cpu(), dataloader))

        model_a = prepare_model(qmodel, mx.cpu(), dataloader.input_desc)
        model_b = prepare_model(qmodel_r, mx.cpu(), dataloader.input_desc)
        are_models_equal(self, model_a, model_b)

        # test gluon model saving
        qmodel_r.save(WORKSPACE_DIR/'save_test')


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tuning class."""

import os
from typing import List, Optional

from neural_compressor.ux.components.optimization.optimization import Optimization
from neural_compressor.ux.utils.workload.workload import Workload


class Tuning(Optimization):
    """Tuning class."""

    def __init__(
        self,
        workload: Workload,
        template_path: Optional[str] = None,
    ) -> None:
        """Initialize Tuning class."""
        super().__init__(workload, template_path)
        if template_path:
            self.command = ["python", template_path]

    def execute(self) -> None:
        """Execute tuning."""
        pass

    @property
    def optimization_script(self) -> str:
        """Get optimization script path."""
        return os.path.join(os.path.dirname(__file__), "tune_model.py")

    @property
    def parameters(self) -> List[str]:
        """Get optimization parameters."""
        return [
            "--input-graph",
            self.input_graph,
            "--output-graph",
            self.output_graph,
            "--config",
            self.config_path,
            "--framework",
            self.framework,
        ]



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""History Snapshot Parser Service."""
from typing import Optional


class HistorySnapshotParser:
    """History Snapshot Parser."""

    def __init__(self, history_snapshot: list, provide_performance: bool = False):
        """Create object."""
        self.history_snapshot = history_snapshot
        self.provide_performance = provide_performance

    def parse_history_snapshot(self) -> dict:
        """Parse provided history snapshot."""
        if not self.history_snapshot:
            return {}

        if 1 != len(self.history_snapshot):
            raise ValueError("Expected history snapshot with one entry only")

        first_item = self.history_snapshot[0]
        return {
            "baseline_accuracy": self.extract_accuracy(first_item.get("baseline")),
            "baseline_performance": self.extract_performance(first_item.get("baseline")),
            "last_tune_accuracy": self.extract_accuracy(first_item.get("last_tune_result")),
            "last_tune_performance": self.extract_performance(first_item.get("last_tune_result")),
            "best_tune_accuracy": self.extract_accuracy(first_item.get("best_tune_result")),
            "best_tune_performance": self.extract_performance(first_item.get("best_tune_result")),
            "history": [
                self.parse_history_item(history_item)
                for history_item in first_item.get("history", [])
            ],
        }

    def parse_history_item(self, history_item: dict) -> dict:
        """Parse single item from history."""
        return {
            "accuracy": self.extract_accuracy(history_item.get("tune_result")),
            "performance": self.extract_performance(history_item.get("tune_result")),
        }

    def extract_accuracy(self, measurements: Optional[tuple]) -> Optional[float]:
        """Extract accuracy metric from result."""
        if not measurements:
            return None
        return measurements[0]

    def extract_performance(self, measurements: Optional[tuple]) -> Optional[float]:
        """Extract performance metric from result if needed."""
        if not self.provide_performance:
            return None
        if not measurements:
            return None
        return measurements[1]



import os
import unittest
import numpy as np
import tensorflow as tf
from neural_compressor.adaptor.tf_utils.util import get_graph_def
from neural_compressor.adaptor.tf_utils.util import collate_tf_preds
from neural_compressor.adaptor.tf_utils.util import fix_ref_type_of_graph_def
from neural_compressor.adaptor.tf_utils.util import disable_random
from tensorflow.core.framework import graph_pb2
from neural_compressor.adaptor.tf_utils.graph_rewriter.graph_util import GraphRewriterHelper as Helper
from tensorflow.python.framework import dtypes

def build_fake_graphdef():
    graph_def = graph_pb2.GraphDef()
    constant_1_name = 'moving_1/switch_input_const'
    constant_1 = Helper.create_constant_node(
            constant_1_name,
            value=0.,
            dtype=dtypes.float32)

    constant_3_name = 'moving_1/switch_input_const/read'
    constant_3 = Helper.create_constant_node(
            constant_3_name,
            value=[1],
            dtype=dtypes.float32)

    constant_2_name = 'switch_input_const2'
    constant_2 = Helper.create_constant_node(
            constant_2_name,
            value=2.,
            dtype=dtypes.float32)
    equal_name = 'equal'
    equal = Helper.create_node("Equal", equal_name, [constant_1_name, constant_2_name])
    Helper.set_attr_dtype(equal, 'T', dtypes.float32)

    refswitch_name = 'refswitch'
    refswitch_node = Helper.create_node("RefSwitch", refswitch_name,
                                                    [constant_1_name ,equal_name])
    Helper.set_attr_dtype(refswitch_node, 'T', dtypes.float32)

    variable_name = 'variable'
    variable_node = Helper.create_node("VariableV2", variable_name,
                                                    [])
    Helper.set_attr_dtype(variable_node, 'T', dtypes.float32)

    assign_name = 'assign'
    assign_node = Helper.create_node("Assign", assign_name,
                                                    [variable_name,refswitch_name])
    Helper.set_attr_bool(assign_node, 'use_locking', True)
    Helper.set_attr_bool(assign_node, 'validate_shape', True)
    Helper.set_attr_dtype(assign_node, 'T', dtypes.float32)

    assignsub_name = 'assignsub'
    assignsub_node = Helper.create_node("AssignSub", assignsub_name,
                                                    [assign_name,constant_1_name])
    Helper.set_attr_bool(assignsub_node, 'use_locking', True)
    Helper.set_attr_dtype(assignsub_node, 'T', dtypes.float32)

    assignadd_name = 'assignadd'
    assignadd_node = Helper.create_node("AssignAdd", assignadd_name,
                                                    [assignsub_name,constant_2_name])
    Helper.set_attr_bool(assignadd_node, 'use_locking', True)
    Helper.set_attr_dtype(assignadd_node, 'T', dtypes.float32)

    graph_def.node.extend([
        constant_1,
        constant_2,
        constant_3,
        equal,
        refswitch_node,
        variable_node,
        assign_node,
        assignsub_node,
        assignadd_node
    ])
    return graph_def

class TestTFutil(unittest.TestCase):

    @classmethod
    def tearDownClass(self):
        os.remove('test.pb')

    @disable_random()
    def test_fix_ref_type(self):
        graph_def = build_fake_graphdef()
        new_graph_def = fix_ref_type_of_graph_def(graph_def)
        f = tf.io.gfile.GFile('./test.pb', 'wb')
        f.write(new_graph_def.SerializeToString())
        find_Assign_prefix = False
        for node in new_graph_def.node:
            if 'Assign' in node.op:
                find_Assign_prefix = True
        self.assertFalse(find_Assign_prefix, False)

    @disable_random()
    def test_collate_tf_preds(self):
        results = [[1],[np.array([2])]]
        data = collate_tf_preds(results)
        self.assertEqual(data,[1,np.array([2])])

    @disable_random()
    def test_get_graph_def(self):
        graphdef = get_graph_def('./test.pb', outputs="assignadd")
        self.assertIsInstance(graphdef, tf.compat.v1.GraphDef)

if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph reader."""


from typing import List

from neural_compressor.ux.components.graph.collapser import Collapser
from neural_compressor.ux.components.graph.graph import Graph
from neural_compressor.ux.components.model.repository import ModelRepository


class GraphReader:
    """Model graph reader."""

    def read(self, model_path: str, expanded_groups: List[str]) -> Graph:
        """Return Graph for given model path."""
        model_repository = ModelRepository()
        model = model_repository.get_model(model_path)
        graph = model.get_model_graph()

        collapser = Collapser(expanded_groups)
        collapsed_graph = collapser.collapse(graph)

        return collapsed_graph



import os
import shutil
import unittest

import torch
import torchvision
import torch.nn as nn

from neural_compressor.experimental.data.datasets.dummy_dataset import DummyDataset
from neural_compressor.experimental.data.dataloaders.pytorch_dataloader import PyTorchDataLoader

def build_fake_yaml():
    fake_yaml = """
    model:
      name: imagenet_prune
      framework: pytorch

    pruning:
      approach:
        weight_compression:
          start_epoch: 0
          pruners:
            - !Pruner
                prune_type: pattern_lock
                names: ['layer1.0.conv1.weight']
    evaluation:
      accuracy:
        metric:
          topk: 1
    """
    with open('fake.yaml', 'w', encoding="utf-8") as f:
        f.write(fake_yaml)


class TestPatternLock(unittest.TestCase):
    model = torchvision.models.resnet18()

    @classmethod
    def setUpClass(cls):
        build_fake_yaml()

    @classmethod
    def tearDownClass(cls):
        os.remove('fake.yaml')
        shutil.rmtree('./saved', ignore_errors=True)
        shutil.rmtree('runs', ignore_errors=True)

    def test_pattern_lock(self):
        from neural_compressor.experimental import Pruning, common
        prune = Pruning('fake.yaml')

        weight = self.model.layer1[0].conv1.weight
        mask = torch.ones(weight.numel())
        mask[:round(weight.numel()*0.9)] = .0
        mask = mask[torch.randperm(mask.numel())].view(weight.shape)
        weight.data = weight * mask

        self.assertTrue(self.model.layer1[0].conv1.weight.ne(0).eq(mask).all())

        dummy_dataset = DummyDataset([tuple([100, 3, 256, 256])])
        dummy_dataloader = PyTorchDataLoader(dummy_dataset)

        def training_func_for_nc(model):
            epochs = 2
            iters = 30
            criterion = nn.CrossEntropyLoss()
            optimizer = torch.optim.SGD(model.parameters(), lr=0.0001)
            for nepoch in range(epochs):
                model.train()
                cnt = 0
                prune.on_epoch_begin(nepoch)
                for i, (image, target) in enumerate(dummy_dataloader):
                    prune.on_batch_begin(cnt)
                    print('.', end='')
                    cnt += 1
                    output = model(image)
                    loss = criterion(output, target)
                    optimizer.zero_grad()
                    loss.backward()
                    optimizer.step()
                    prune.on_batch_end()
                    if cnt >= iters:
                        break
                prune.on_epoch_end()
        dummy_dataset = DummyDataset(tuple([100, 3, 256, 256]), label=True)
        dummy_dataloader = PyTorchDataLoader(dummy_dataset)
        prune.model = common.Model(self.model)
        prune.pruning_func = training_func_for_nc
        prune.eval_dataloader = dummy_dataloader
        prune.train_dataloader = dummy_dataloader
        _ = prune()
        self.assertTrue(self.model.layer1[0].conv1.weight.ne(0).eq(mask).all())


if __name__ == "__main__":
    unittest.main()



"""Tests for quantization"""
import numpy as np
import unittest
import os
import shutil
import yaml
import tensorflow as tf

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            strategy:
              name: tpe
            accuracy_criterion:
              relative: 0.01
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml',"w",encoding="utf-8") as f:
        yaml.dump(y,f)
    f.close()

def build_fake_yaml2():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
          strategy:
            name: tpe
          exit_policy:
            max_trials: 5
          accuracy_criterion:
            relative: -0.01
          workspace:
            path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml2.yaml',"w",encoding="utf-8") as f:
        yaml.dump(y,f)
    f.close()

def build_fake_model():
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()

        with tf.Session() as sess:
            x = tf.placeholder(tf.float32, shape=(1,3,3,1), name='x')
            y = tf.constant(np.random.random((2,2,1,1)), name='y', dtype=tf.float32)
            op = tf.nn.conv2d(input=x, filter=y, strides=[1,1,1,1], padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session() as sess:
            x = tf.compat.v1.placeholder(tf.float32, shape=(1,3,3,1), name='x')
            y = tf.compat.v1.constant(np.random.random((2,2,1,1)), name='y', dtype=tf.float32)
            op = tf.nn.conv2d(input=x, filters=y, strides=[1,1,1,1], padding='VALID', name='op_to_store')

            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    return graph

class TestQuantization(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.constant_graph = build_fake_model()
        build_fake_yaml()
        build_fake_yaml2()

    @classmethod
    def tearDownClass(self):
        try:
            os.remove('fake_yaml.yaml')
            os.remove('fake_yaml2.yaml')

            shutil.rmtree("saved", ignore_errors=True)
        except:
            print("Error while deleting file ")

    def test_run_tpe_one_trial(self):
        from neural_compressor.experimental import Quantization, common

        quantizer = Quantization('fake_yaml.yaml')
        dataset = quantizer.dataset('dummy', (100, 3, 3, 1), label=True)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.model = self.constant_graph
        quantizer()

    def test_run_tpe_max_trials(self):
        from neural_compressor.experimental import Quantization, common

        quantizer = Quantization('fake_yaml2.yaml')
        dataset = quantizer.dataset('dummy', (100, 3, 3, 1), label=True)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.model = self.constant_graph
        quantizer()

    def test_loss_calculation(self):
        from neural_compressor.contrib.strategy.tpe import TpeTuneStrategy
        from neural_compressor.experimental import Quantization, common

        quantizer = Quantization('fake_yaml.yaml')
        dataset = quantizer.dataset('dummy', (100, 3, 3, 1), label=True)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.model = self.constant_graph

        testObject = TpeTuneStrategy(quantizer.model, quantizer.conf, quantizer.calib_dataloader)
        testObject._calculate_loss_function_scaling_components(0.01, 2, testObject.loss_function_config)
        # check if latency difference between min and max corresponds to 10 points of loss function
        tmp_val = testObject.calculate_loss(0.01, 2, testObject.loss_function_config)
        tmp_val2 = testObject.calculate_loss(0.01, 1, testObject.loss_function_config)
        self.assertTrue(True if int(tmp_val2 - tmp_val) == 10 else False)
        # check if 1% of acc difference corresponds to 10 points of loss function
        tmp_val = testObject.calculate_loss(0.02, 2, testObject.loss_function_config)
        tmp_val2 = testObject.calculate_loss(0.03, 2, testObject.loss_function_config)
        self.assertTrue(True if int(tmp_val2 - tmp_val) == 10 else False)

if __name__ == "__main__":
    unittest.main()



from io import open
from setuptools import find_packages, setup, Extension
from setuptools.command.build_ext import build_ext
import os
import re
import sys
import shutil

cwd = os.path.dirname(os.path.abspath(__file__))

try:
    filepath = './neural_compressor/version.py'
    with open( filepath ) as version_file:
        __version__ ,= re.findall( '__version__ = "(.*)"', version_file.read() )
except Exception as error:
    assert False,  "Error: Could not open '%s' due %s\n" % (filepath, error)

class build_ext(build_ext):
    def build_extension(self, ext):
        if not sys.platform.startswith("win"):
            import pathlib
            cwd = pathlib.Path().absolute()
    
            build_temp = pathlib.Path(self.build_temp)
            build_temp.mkdir(parents=True, exist_ok=True)
            extdir = pathlib.Path(self.get_ext_fullpath(ext.name))
            executable_path = extdir.parent.absolute()
            cmake_args = [
                '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + str(extdir.parent.absolute()),
                '-DPYTHON_EXECUTABLE={}'.format(sys.executable)
            ]
    
            build_args = [
                '-j'
            ]
    
            os.chdir(str(build_temp))
            self.spawn(['cmake3', ext.sourcedir] + cmake_args)
            self.spawn(['make'] + build_args)
            if os.path.exists('inferencer'):
                shutil.copy('inferencer', executable_path)
            os.chdir(str(cwd))
        else:
            print("Engine is not support windows for now")

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

def check_submodules():
    def check_for_files(folder, files):
        if not any(os.path.exists(os.path.join(folder, f)) for f in files):
            report("Could not find any of {} in {}".format(", ".join(files), folder))
            report("Did you run 'git submodule update --init --recursive'?")
            sys.exit(1)

    def not_exists_or_empty(folder):
        return not os.path.exists(folder) or (os.path.isdir(folder) and len(os.listdir(folder)) == 0)

    git_modules_path = os.path.join(cwd, ".gitmodules")
    with open(git_modules_path) as f:
        folders = [os.path.join(cwd, line.split("=", 1)[1].strip()) for line in
                   f.readlines() if line.strip().startswith("path")]

    # If none of the submodule folders exists, try to initialize them
    if all(not_exists_or_empty(folder) for folder in folders) and not sys.platform.startswith("win"):
        try:
            print(' --- Trying to initialize submodules')
            start = time.time()
            subprocess.check_call(["git", "submodule", "update", "--init", "--recursive"], cwd=cwd)
            end = time.time()
            print(' --- Submodule initialization took {:.2f} sec'.format(end - start))
        except Exception:
            print(' --- Submodule initalization failed')
            print('Please run:\n\tgit submodule update --init --recursive')
            sys.exit(1)

if __name__ == '__main__':
    check_submodules()

    setup(
        name="neural_compressor",
        version=__version__,
        author="Intel MLP/MLPC Team",
        author_email="feng.tian@intel.com, chuanqi.wang@intel.com, pengxin.yuan@intel.com, guoming.zhang@intel.com, haihao.shen@intel.com, jiong.gong@intel.com, xi2.chen@intel.com",
        description="Repository of Intel® Neural Compressor",
        long_description=open("README.md", "r", encoding='utf-8').read(),
        long_description_content_type="text/markdown",
        keywords='quantization, auto-tuning, post-training static quantization, post-training dynamic quantization, quantization-aware training, tuning strategy',
        license='',
        url="https://github.com/intel/neural-compressor",
        ext_modules=[CMakeExtension("engine_py", str(cwd) + '/engine/executor/')],
        packages = find_packages(),
        include_package_data = True,
        package_dir = {'':'.'},
        package_data={
            '': ['*.py', '*.yaml'],
            'neural_compressor.ux': [
                "web/static/*.*",
                "web/static/assets/*.*",
                "web/static/assets/fonts/*.*",
                "utils/configs/*.json",
                "utils/configs/predefined_configs/**/*.yaml",
                "utils/templates/*.txt",
            ],
            'engine': ['*.py'],
        },
        cmdclass={
            'build_ext': build_ext,
        },
        install_requires=[
            'numpy', 'pyyaml', 'scikit-learn', 'schema', 'py-cpuinfo', 'hyperopt', 'pandas', 'pycocotools', 'opencv-python',
            'requests', 'Flask-Cors', 'Flask-SocketIO', 'Flask', 'gevent-websocket', 'gevent', 'psutil', 'Pillow', 'sigopt',
            'prettytable'],
        scripts=['neural_compressor/ux/bin/neural_compressor_bench', 'engine/bin/inferencer'],
        python_requires='>=3.6.0',
        classifiers=[
              'Intended Audience :: Science/Research',
              'Programming Language :: Python :: 3',
              'Topic :: Scientific/Engineering :: Artificial Intelligence',
        ],
    )



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Configuration graph_optimization module."""

from typing import Any, Dict, List, Optional, Union

from neural_compressor.ux.utils.exceptions import ClientErrorException
from neural_compressor.ux.utils.json_serializer import JsonSerializer


class GraphOptimization(JsonSerializer):
    """Configuration Graph Optimization class."""

    def __init__(self, data: Dict[str, Any] = {}) -> None:
        """Initialize Configuration Graph Optimization class."""
        super().__init__()
        self.precisions: Optional[str] = None
        if data.get("precisions"):
            self.set_precisions(data.get("precisions"))  # type: ignore

        self.op_wise = None
        if isinstance(data.get("op_wise"), dict):
            self.op_wise = data.get("op_wise", {})

    def set_precisions(self, precisions: Union[str, List[str]]) -> None:
        """Update graph_optimization precisions in config."""
        if isinstance(precisions, str):
            self.precisions = precisions.replace(" ", "")
        elif isinstance(precisions, list):
            self.precisions = ",".join(precision.strip() for precision in precisions)
        else:
            raise ClientErrorException(
                "Precision should be comma separated string with "
                "precisions or list of string precisions",
            )



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Configuration module for UX server."""

import argparse
import json
import logging
import os
import secrets
import socket
from typing import Dict

from numpy.random import randint

from neural_compressor.utils.utility import singleton
from neural_compressor.ux.utils.exceptions import NotFoundException


@singleton
class Configuration:
    """Configuration object for UX server."""

    PORT_DEFAULT = 5000
    MAX_PORTS_TRIED = 10

    def __init__(self) -> None:
        """Set the variables."""
        self.server_address = ""
        self.server_port = 0
        self.gui_port = 0
        self.log_level = 0
        self.token = ""
        self.scheme = ""
        self.workdir = ""
        self.set_up()

    def set_up(self) -> None:
        """Reset variables."""
        self.determine_values_from_environment()
        self.determine_values_from_existing_config()

    def determine_values_from_environment(self) -> None:
        """Set variables based on environment values."""
        self.server_address = "localhost"
        args = self.get_command_line_args()
        self.server_port = self.determine_server_port(args)
        self.gui_port = self.determine_gui_port(args)
        self.log_level = self.determine_log_level(args)
        self.token = secrets.token_hex(16)
        self.scheme = "http"
        self.workdir = os.path.join(os.environ.get("HOME", ""), "workdir")

    def determine_values_from_existing_config(self) -> None:
        """Set variables based on existing files."""
        workloads_list_filepath = os.path.join(
            os.environ.get("HOME", ""),
            ".neural_compressor",
            "workloads_list.json",
        )
        if os.path.isfile(workloads_list_filepath):
            with open(workloads_list_filepath, encoding="utf-8") as workloads_list:
                workloads_data = json.load(workloads_list)
                self.workdir = workloads_data.get("active_workspace_path", self.workdir)

    def get_command_line_args(self) -> Dict:
        """Return arguments passed in command line."""
        parser = argparse.ArgumentParser(
            description="Run Intel(r) Neural Compressor Bench server.",
        )
        parser.add_argument(
            "-p",
            "--port",
            type=int,
            help="server port number to listen on",
        )
        parser.add_argument(
            "-P",
            "--gui_port",
            type=int,
            help="port number for GUI",
        )
        parser.add_argument(
            "--verbose",
            "-v",
            action="count",
            default=0,
            help="verbosity of logging output, use -vv and -vvv for even more logs",
        )
        return vars(parser.parse_args())

    def determine_server_port(self, args: Dict) -> int:
        """
        Return port to be used by the server.

        Will raise a NotFoundException if port is already in use.

        When port given in command line, only that port will be tried.
        When no port specified will try self.MAX_PORTS_TRIED times,
        starting with self.PORT_DEFAULT.
        """
        command_line_port = args.get("port")
        if command_line_port is not None:
            self._ensure_valid_port(command_line_port)
            if self.is_port_taken(command_line_port):
                raise NotFoundException(
                    f"Port {command_line_port} already in use, exiting.",
                )
            else:
                return command_line_port

        ports = [self.PORT_DEFAULT] + randint(
            1025,
            65536,
            self.MAX_PORTS_TRIED - 1,
        ).tolist()

        for port in ports:
            if not self.is_port_taken(port):
                return port

        raise NotFoundException(
            f"Unable to find a free port in {len(ports)} attempts, exiting.",
        )

    def determine_gui_port(self, args: Dict) -> int:
        """
        Return port to be used by the GUI client.

        Will return self.server_port unless specified in configuration.
        """
        command_line_port = args.get("gui_port")
        if command_line_port is not None:
            self._ensure_valid_port(command_line_port)
            return command_line_port
        return self.server_port

    def is_port_taken(self, port: int) -> bool:
        """Return if given port is already in use."""
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        try:
            s.bind((self.server_address, port))
        except socket.error:
            return True
        finally:
            s.close()

        return False

    def determine_log_level(self, args: Dict) -> int:
        """Determine log level based on parameters given."""
        verbosity_mapping = [
            logging.CRITICAL,
            logging.WARNING,
            logging.INFO,
            logging.DEBUG,
        ]
        verbosity: int = args.get("verbose")  # type:ignore
        try:
            return verbosity_mapping[verbosity]
        except IndexError:
            return logging.DEBUG

    def get_url(self) -> str:
        """Return URL to access application."""
        return f"{self.scheme}://{self.server_address}:{self.gui_port}/?token={self.token}"

    def _ensure_valid_port(self, port: int) -> None:
        """Validate if proposed port number is allowed by TCP/IP."""
        if port < 1:
            raise ValueError(f"Lowest allowed port number is 1, attempted to use: {port}")
        if port > 65535:
            raise ValueError(f"Highest allowed port number is 65535, attempted to use: {port}")



import unittest
import shutil
import tensorflow as tf
from tensorflow import keras


def train_func():
    # Load MNIST dataset
    mnist = keras.datasets.mnist
    (train_images, train_labels), (test_images, test_labels) = mnist.load_data()

    # Normalize the input image so that each pixel value is between 0 to 1.
    train_images = train_images / 255.0
    test_images = test_images / 255.0

    # Define the model architecture.
    model = keras.Sequential([
        keras.layers.InputLayer(input_shape=(28, 28)),
        keras.layers.Reshape(target_shape=(28, 28, 1)),
        keras.layers.Conv2D(filters=12, kernel_size=(3, 3), activation='relu'),
        keras.layers.MaxPooling2D(pool_size=(2, 2)),
        keras.layers.Flatten(),
        keras.layers.Dense(10)
    ])
    # Train the digit classification model
    model.compile(optimizer='adam',
                  loss=tf.keras.losses.SparseCategoricalCrossentropy(
                      from_logits=True),
                  metrics=['accuracy'])

    model.fit(
        train_images,
        train_labels,
        epochs=1,
        validation_split=0.1,
    )

    model.save("baseline_model")

class TestTensorflowPruning(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        train_func()

    @classmethod
    def tearDownClass(self):
        shutil.rmtree('baseline_model',ignore_errors=True)
    
    @unittest.skipIf(tf.version.VERSION < '2.3.0', " keras model need tensorflow version >= 2.3.0, so the case is skipped")
    def test_pruning_utility(self):
        from neural_compressor.experimental import common
        pruning_model = common.Model("baseline_model")
        all_weights_name = pruning_model.get_all_weight_names()
        df, sparsity = pruning_model.report_sparsity()
        self.assertEqual(all_weights_name, [1, 4])
        self.assertEqual(df.empty, False)
        self.assertNotEqual(sparsity, None)

if __name__ == '__main__':
    unittest.main()
